using System;
using System.Text;
using System.Web;
using System.Web.UI.HtmlControls;
using System.IO;
using System.Web.UI;

using CMS.UIControls;
using CMS.GlobalHelper;
using CMS.ExtendedControls;
using CMS.TreeEngine;
using CMS.FileManager;
using CMS.CMSHelper;
using CMS.WorkflowEngine;
using CMS.LicenseProvider;
using CMS.SettingsProvider;
using CMS.FormEngine;
using CMS.SiteProvider;

public partial class CMSModules_Content_Controls_Attachments_DirectFileUploader_DirectFileUploaderControl : DirectFileUploader
{
    #region "Variables"

    private bool? mIsSupportedBrowser = null;

    protected TreeNode node = null;
    protected TreeProvider mTreeProvider = null;
    private WorkflowManager mWorkflowManager = null;
    private WorkflowInfo wi = null;
    private VersionManager mVersionManager = null;

    #endregion


    #region "Public properties"

    /// <summary>
    /// Uploaded file
    /// </summary>
    public override HttpPostedFile PostedFile
    {
        get
        {
            return ucFileUpload.PostedFile;
        }
    }


    /// <summary>
    /// File upload user control.
    /// </summary>
    public override CMSFileUpload FileUploadControl
    {
        get
        {
            return ucFileUpload;
        }
    }


    /// <summary>
    /// Indicates if supported browser.
    /// </summary>
    public bool IsSupportedBrowser
    {
        get
        {
            if (mIsSupportedBrowser == null)
            {
                string browser = DataHelper.GetNotEmpty(CMSContext.GetBrowserClass(), "").ToLowerInvariant();
                mIsSupportedBrowser = browser.Contains("ie") || browser.Contains("gecko");
            }
            return mIsSupportedBrowser.Value;
        }
    }

    #endregion


    #region "Protected properties"

    /// <summary>
    /// Indicates if check-in/check-out functionality is automatic
    /// </summary>
    protected bool AutoCheck
    {
        get
        {
            if (node != null)
            {
                // Get workflow info
                wi = WorkflowManager.GetNodeWorkflow(node);

                // Check if the document uses workflow
                if (wi != null)
                {
                    return !wi.UseCheckInCheckOut(CMSContext.CurrentSiteName);
                }
            }
            return false;
        }
    }


    /// <summary>
    /// Gets Workflow manager instance
    /// </summary>
    protected WorkflowManager WorkflowManager
    {
        get
        {
            return mWorkflowManager ?? (mWorkflowManager = new WorkflowManager(TreeProvider));
        }
    }


    /// <summary>
    /// Gets Version manager instance
    /// </summary>
    protected VersionManager VersionManager
    {
        get
        {
            return mVersionManager ?? (mVersionManager = new VersionManager(TreeProvider));
        }
    }


    /// <summary>
    /// Tree provider instance
    /// </summary>
    protected TreeProvider TreeProvider
    {
        get
        {
            return mTreeProvider ?? (mTreeProvider = new TreeProvider(CMSContext.CurrentUser, Connection));
        }
    }

    #endregion


    #region "Page events"

    protected void Page_Load(object sender, EventArgs e)
    {
        if (StopProcessing)
        {
            // Do nothing
            Visible = false;
        }
        else
        {
            ScriptHelper.RegisterScriptFile(Page, "~/CMSModules/Content/Controls/Attachments/DirectFileUploader/DirectFileUploader.js");

            // Prepare disabled image
            // Prepare loading image
            imgLoading.ImageUrl = ResolveUrl(LoadingImageUrl);
            lblLoading.CssClass = InnerLoadingDivClass;
            lblLoading.Text = InnerLoadingDivHtml;
            pnlLoading.Style.Add("display", "none");
            bool isRTL = (IsLiveSite && CultureHelper.IsPreferredCultureRTL()) || (!IsLiveSite && CultureHelper.IsUICultureRTL());
            if (isRTL)
            {
                pnlLoading.Style.Add("text-align", "right");
                pnlStandard.Style.Add("text-align", "right");
            }

            // Hide uploader if browser is unsupported
            if (!IsSupportedBrowser)
            {
                plcUploader.Visible = false;
                plcStandard.Visible = true;
                imgStandard.ImageUrl = ResolveUrl(ImageUrl);

                lblStandard.Text = InnerDivHtml;
                lblStandard.CssClass = InnerDivClass;
                if (!string.IsNullOrEmpty(ImageUrlOver))
                {
                    lblStandard.Attributes.Add("onmouseover", "Hover('" + imgStandard.ClientID + "', '" + ResolveUrl(ImageUrlOver) + "', true);");
                    lblStandard.Attributes.Add("onmouseout", "Hover('" + imgStandard.ClientID + "', '" + ResolveUrl(ImageUrl) + "', false);");

                    StringBuilder hoverScript = new StringBuilder();
                    hoverScript.AppendLine("function Hover(id, imgUrl, over)");
                    hoverScript.AppendLine("{");
                    hoverScript.AppendLine("    var elem = document.getElementById(id);");
                    hoverScript.AppendLine("    if(over)");
                    hoverScript.AppendLine("    {");
                    hoverScript.AppendLine("        elem.setAttribute('src', imgUrl);");
                    hoverScript.AppendLine("    }");
                    hoverScript.AppendLine("    else");
                    hoverScript.AppendLine("    {");
                    hoverScript.AppendLine("        elem.setAttribute('src', imgUrl);");
                    hoverScript.AppendLine("    }");
                    hoverScript.AppendLine("}");

                    ScriptHelper.RegisterClientScriptBlock(this, typeof(string), "uploaderHover", ScriptHelper.GetScript(hoverScript.ToString()));
                }
                // Register dialog script
                ScriptHelper.RegisterClientScriptBlock(this, typeof(string), ScriptHelper.DIALOG_SCRIPT_KEY, ScriptHelper.DialogScript);
                string modalDialog = "modalDialog('" + CMSContext.ResolveDialogUrl(GetDialogUrl()) + "', 'imageEditorDialog', 300, 180);";
                imgStandard.Attributes.Add("onclick", modalDialog);
                lblStandard.Attributes.Add("onclick", modalDialog);
            }
            else
            {
                // Initialize uploader
                Page.Header.Controls.Add(new LiteralControl(CreateCss()));
                HtmlGenericControl lblContainer = (HtmlGenericControl)FindControl("container");
                lblContainer.Attributes.Add("class", "container");
                ucFileUpload.Attributes.Add("class", "fileUpload");
                ucFileUpload.Attributes.Add("onchange", "document.getElementById('" + pnlLoading.ClientID + "').style.display='';document.getElementById('" + lblContainer.ClientID + "').style.display='none';" + Page.ClientScript.GetPostBackEventReference(btnHidden, null, false));

                // SI script
                string SIScript = "var uploaderInitilaized=false;function InitUploader(){if((window.SI) && (!uploaderInitilaized)){ uploaderInitilaized=true;SI.Files.stylizeById('" + ucFileUpload.ClientID + "');}}InitUploader();";
                ScriptHelper.RegisterStartupScript(this, typeof(string), "SIScript_" + ucFileUpload.ClientID, ScriptHelper.GetScript(SIScript));

                string rtl = (isRTL) ? "var rtl = true;" : "var rtl = false;";
                ltlInnerDiv.Text = ScriptHelper.GetScript("var innerDivHtml = '" + InnerDivHtml.Replace("'", "\\\'") + "';\nvar innerDivClass='" + InnerDivClass.Replace("'", "\\\'") + "';" + rtl);
                btnHidden.Attributes.Add("style", "display:none;");
            }
        }
    }


    protected void btnHidden_Click(object sender, EventArgs e)
    {
        if (StopProcessing)
        {
            // Do nothing
        }
        else
        {
            switch (SourceType)
            {
                case MediaSourceEnum.Attachment:
                    HandleAttachmentUpload(true);
                    break;

                case MediaSourceEnum.DocumentAttachments:
                    HandleAttachmentUpload(false);
                    break;

                case MediaSourceEnum.Content:
                    HandleContentUpload();
                    break;

                default:
                    break;
            }
        }
    }

    #endregion


    #region "Attachments"

    /// <summary>
    /// Provides operations necessary to create and store new attachment.
    /// </summary>
    private void HandleAttachmentUpload(bool fieldAttachment)
    {
        // New attachment
        AttachmentInfo newAttachment = null;

        string message = string.Empty;
        bool fullRefresh = false;

        try
        {
            // Get the existing document
            if (DocumentID != 0)
            {
                // Get document
                node = DocumentHelper.GetDocument(DocumentID, TreeProvider);
                if (node == null)
                {
                    throw new Exception("Given document doesn't exist!");
                }
            }

            #region "Check permissions"

            if (CheckPermissions)
            {
                CheckNodePermissions(node);
            }

            #endregion

            // Check the allowed extensions
            CheckAllowedExtensions();

            // Standard attachments
            if (DocumentID != 0)
            {
                // Check out the document
                if (AutoCheck)
                {
                    // Get current step info
                    WorkflowStepInfo si = WorkflowManager.GetStepInfo(node);
                    if (si != null)
                    {
                        // Decide if full refresh is needed
                        bool automaticPublish = wi.WorkflowAutoPublishChanges;
                        string stepName = si.StepName.ToLower();
                        fullRefresh = (stepName == "published") || (stepName == "archived") || (automaticPublish && (stepName != "published"));
                    }

                    VersionManager.CheckOut(node);
                }

                // Handle field attachment
                if (fieldAttachment)
                {
                    newAttachment = DocumentHelper.AddAttachment(node, AttachmentGUIDColumnName, Guid.Empty, Guid.Empty, ucFileUpload.PostedFile, TreeProvider, ResizeToWidth, ResizeToHeight, ResizeToMaxSideSize);
                    // Update attachment field
                    DocumentHelper.UpdateDocument(node, TreeProvider);
                }
                // Handle grouped and unsorted attachments
                else
                {
                    // Grouped attachment
                    if (AttachmentGroupGUID != Guid.Empty)
                    {
                        newAttachment = DocumentHelper.AddGroupedAttachment(node, AttachmentGUID, AttachmentGroupGUID, ucFileUpload.PostedFile, TreeProvider, ResizeToWidth, ResizeToHeight, ResizeToMaxSideSize);
                    }
                    // Unsorted attachment
                    else
                    {
                        newAttachment = DocumentHelper.AddUnsortedAttachment(node, AttachmentGUID, ucFileUpload.PostedFile, TreeProvider, ResizeToWidth, ResizeToHeight, ResizeToMaxSideSize);
                    }

                    // Log synchronization task if not under workflow
                    if (wi == null)
                    {
                        DocumentHelper.LogSynchronization(node, CMS.Staging.TaskTypeEnum.UpdateDocument, TreeProvider);
                    }
                }

                // Check in the document
                if (AutoCheck)
                {
                    VersionManager.CheckIn(node, null, null);
                }
            }

            // Temporary attachments
            if (FormGUID != Guid.Empty)
            {
                newAttachment = AttachmentManager.AddTemporaryAttachment(FormGUID, AttachmentGUIDColumnName, AttachmentGUID, AttachmentGroupGUID, ucFileUpload.PostedFile, CMSContext.CurrentSiteID, ResizeToWidth, ResizeToHeight, ResizeToMaxSideSize);
            }

            // Ensure properties update
            if ((newAttachment != null) && !InsertMode)
            {
                AttachmentGUID = newAttachment.AttachmentGUID;
            }

            if (newAttachment == null)
            {
                throw new Exception("The attachment hasn't been created since no DocumentID or FormGUID was supplied.");
            }
        }
        catch (Exception ex)
        {
            message = ex.Message;
        }
        finally
        {
            string afterSaveScript = string.Empty;
            // Call aftersave javascript if exists
            if (!String.IsNullOrEmpty(AfterSaveJavascript))
            {
                if ((message == string.Empty) && (newAttachment != null))
                {
                    string url = null;
                    string saveName = UrlHelper.GetSafeFileName(newAttachment.AttachmentName, CMSContext.CurrentSiteName);
                    if (node != null)
                    {
                        SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                        if (si != null)
                        {
                            bool usePermanent = SettingsKeyProvider.GetBoolValue(si.SiteName + ".CMSUsePermanentURLs");
                            if (usePermanent)
                            {
                                url = ResolveUrl(AttachmentManager.GetAttachmentUrl(newAttachment.AttachmentGUID, saveName));
                            }
                            else
                            {
                                url = ResolveUrl(AttachmentManager.GetAttachmentUrl(saveName, node.NodeAliasPath));
                            }
                        }
                    }
                    else
                    {
                        url = ResolveUrl(AttachmentManager.GetAttachmentUrl(newAttachment.AttachmentGUID, saveName));
                    }
                    // Calling javascript function with parameters attachments url, name, width, height
                    if (!string.IsNullOrEmpty(AfterSaveJavascript))
                    {
                        string jsParams = "('" + url + "', '" + newAttachment.AttachmentName + "', '" + newAttachment.AttachmentImageWidth + "', '" + newAttachment.AttachmentImageHeight + "');";
                        string script = "if (window." + AfterSaveJavascript + " != null){window." + AfterSaveJavascript + jsParams + "}";
                        script += "else if ((window.parent != null) && (window.parent." + AfterSaveJavascript + " != null)){window.parent." + AfterSaveJavascript + jsParams + "}";
                        afterSaveScript += ScriptHelper.GetScript(script);
                    }
                }
                else
                {
                    afterSaveScript += ScriptHelper.GetAlertScript(message);
                }
            }

            // Create attachment info string
            string attachmentInfo = "";
            if ((newAttachment != null) && (newAttachment.AttachmentGUID != Guid.Empty) && (IncludeNewItemInfo))
            {
                attachmentInfo = newAttachment.AttachmentGUID.ToString();
            }

            // Ensure message text
            message = HTMLHelper.EnsureLineEnding(message, " ");

            // Call function to refresh parent window
            afterSaveScript += ScriptHelper.GetScript("if ((window.parent != null) && (/parentelemid=" + ParentElemID + "/i.test(window.location.href)) && (window.parent.InitRefresh_" + ParentElemID + " != null)){window.parent.InitRefresh_" + ParentElemID + "(" + ScriptHelper.GetString(message.Trim()) + ", " + (fullRefresh ? "true" : "false") + ((attachmentInfo != "") ? ", '" + attachmentInfo + "'" : "") + (InsertMode ? ", 'insert'" : ", 'update'") + ");}");

            ScriptHelper.RegisterStartupScript(this, typeof(string), "afterSaveScript_" + ClientID, afterSaveScript);
        }
    }


    /// <summary>
    /// Check permissions
    /// </summary>
    /// <param name="node">Tree node</param>
    private void CheckNodePermissions(TreeNode node)
    {
        // For new document
        if (FormGUID != Guid.Empty)
        {
            if (NodeParentNodeID == 0)
            {
                throw new Exception(ResHelper.GetString("attach.document.parentmissing"));
            }

            if (!RaiseOnCheckPermissions("Create", this))
            {
                if (!CMSContext.CurrentUser.IsAuthorizedToCreateNewDocument(NodeParentNodeID, NodeClassName))
                {
                    throw new Exception(ResHelper.GetString("attach.actiondenied"));
                }
            }
        }
        // For existing document
        else if (DocumentID > 0)
        {
            if (!RaiseOnCheckPermissions("Modify", this))
            {
                if (CMSContext.CurrentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
                {
                    throw new Exception(ResHelper.GetString("attach.actiondenied"));
                }
            }
        }
    }

    #endregion


    #region "Content files"

    /// <summary>
    /// Provides operations necessary to create and store new content file.
    /// </summary>
    private void HandleContentUpload()
    {
        string message = string.Empty;
        bool newDocumentCreated = false;

        try
        {
            if (NodeParentNodeID == 0)
            {
                throw new Exception(ResHelper.GetString("dialogs.document.parentmissing"));
            }

            // Check license limitations
            if (!LicenseHelper.LicenseVersionCheck(UrlHelper.GetCurrentDomain(), FeatureEnum.Documents, VersionActionEnum.Insert))
            {
                throw new Exception(ResHelper.GetString("cmsdesk.documentslicenselimits"));
            }

            // Check if class exists
            DataClassInfo ci = DataClassInfoProvider.GetDataClass("CMS.File");
            if (ci == null)
            {
                throw new Exception(string.Format(ResHelper.GetString("dialogs.newfile.classnotfound"), "CMS.File"));
            }

            #region "Check permissions"

            // Get the node

            TreeNode parentNode = TreeProvider.SelectSingleNode(NodeParentNodeID, CMSContext.PreferredCultureCode, true);
            if (parentNode != null)
            {
                if (!DataClassInfoProvider.IsChildClassAllowed(ValidationHelper.GetInteger(parentNode.GetValue("NodeClassID"), 0), ci.ClassID))
                {
                    throw new Exception(ResHelper.GetString("Content.ChildClassNotAllowed"));
                }
            }

            // Check user permissions
            if (!RaiseOnCheckPermissions("Create", this))
            {
                if (!CMSContext.CurrentUser.IsAuthorizedToCreateNewDocument(parentNode, "CMS.File"))
                {
                    throw new Exception(string.Format(ResHelper.GetString("dialogs.newfile.notallowed"), "CMS.File"));
                }
            }

            #endregion

            // Check the allowed extensions
            CheckAllowedExtensions();

            // Create new document
            string fileName = Path.GetFileNameWithoutExtension(ucFileUpload.FileName);
            string ext = Path.GetExtension(ucFileUpload.FileName);

            // Make sure the file name respects maximum file name limit porvided by the database column (100 characters)
            if (fileName.Length >= 100)
            {
                fileName = TextHelper.LimitLength(fileName, 90, "");
                fileName = fileName + ext;
            }

            node = new TreeNode("CMS.File", TreeProvider);
            node.DocumentCulture = CMSContext.PreferredCultureCode;
            node.DocumentName = fileName;
            if (NodeGroupID > 0)
            {
                node.SetValue("NodeGroupID", NodeGroupID);
            }

            // Load default values
            FormHelper.LoadDefaultValues(node);

            node.SetValue("FileDescription", "");
            node.SetValue("FileName", fileName);
            node.SetValue("FileAttachment", Guid.Empty);

            // Set default template ID                
            node.DocumentPageTemplateID = ci.ClassDefaultPageTemplateID;

            // Insert the document
            DocumentHelper.InsertDocument(node, parentNode.NodeID, TreeProvider);

            newDocumentCreated = true;

            // Add the file
            DocumentHelper.AddAttachment(node, "FileAttachment", ucFileUpload.PostedFile, TreeProvider, ResizeToWidth, ResizeToHeight, ResizeToMaxSideSize);

            // Create default SKU if configured
            if (ModuleEntry.CheckModuleLicense(ModuleEntry.ECOMMERCE, UrlHelper.GetCurrentDomain(), FeatureEnum.Ecommerce, VersionActionEnum.Insert))
            {
                node.CreateDefaultSKU();
            }

            // Update the document
            DocumentHelper.UpdateDocument(node, TreeProvider);

            // Get workflow info
            wi = WorkflowManager.GetNodeWorkflow(node);

            // Check if auto publish changes is allowed
            if ((wi != null) && wi.WorkflowAutoPublishChanges && !wi.UseCheckInCheckOut(CMSContext.CurrentSiteName))
            {
                // Automatically publish document
                WorkflowManager.AutomaticallyPublish(node, wi, null);
            }
        }
        catch (Exception ex)
        {
            // Delete the document if something failed
            if (newDocumentCreated && (node != null) && (node.DocumentID > 0))
            {
                DocumentHelper.DeleteDocument(node, TreeProvider, false, true, true);
            }

            message = ex.Message;
        }
        finally
        {
            // Create node info string
            string nodeInfo = "";
            if ((node != null) && (node.NodeID > 0) && (IncludeNewItemInfo))
            {
                nodeInfo = node.NodeID.ToString();
            }

            // Ensure message text
            message = HTMLHelper.EnsureLineEnding(message, " ");

            // Call function to refresh parent window                                                     
            string afterSaveScript = null;
            if (!string.IsNullOrEmpty(AfterSaveJavascript))
            {
                afterSaveScript += "if (window." + AfterSaveJavascript + " != null){window." + AfterSaveJavascript + "();}";
                afterSaveScript += "else if ((window.parent != null) && (window.parent." + AfterSaveJavascript + " != null)){window.parent." + AfterSaveJavascript + "();}";
            }

            afterSaveScript += "if ((window.parent != null) && (/parentelemid=" + ParentElemID + "/i.test(window.location.href)) && (window.parent.InitRefresh_" + ParentElemID + " != null)){window.parent.InitRefresh_" + ParentElemID + "(" + ScriptHelper.GetString(message.Trim()) + ", false" + ((nodeInfo != "") ? ", '" + nodeInfo + "'" : "") + (InsertMode ? ", 'insert'" : ", 'update'") + ");}";
            ScriptHelper.RegisterStartupScript(this, typeof(string), "afterSaveScript_" + ClientID, ScriptHelper.GetScript(afterSaveScript));
        }
    }

    #endregion
}
