using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.EmailEngine;
using CMS.DataEngine;
using CMS.SiteProvider;
using CMS.FileManager;
using CMS.UIControls;

public partial class CMSSiteManager_Development_EmailTemplates_EmailTemplate_Edit : SiteManagerPage
{
    #region "Variables"

    protected int templateId = 0;
    protected int siteId = 0;

    #endregion


    #region "Methods"

    /// <summary>
    /// Page load.
    /// </summary>
    protected void Page_Load(object sender, EventArgs e)
    {
        // Initialize required field validators
        RequiredFieldValidatorDisplayName.ErrorMessage = ResHelper.GetString("EmailTemplate_Edit.FillDisplayNameField");
        RequiredFieldValidatorCodeName.ErrorMessage = ResHelper.GetString("EmailTemplate_Edit.FillCodeNameField");
        
        // Initialize other controls
        lblDisplayName.ResourceString = "General.DisplayName";
        lblDisplayName.ShowRequiredMark = true;
        lblDisplayName.DisplayColon = true;
        lblCodeName.ResourceString = "General.CodeName";
        lblCodeName.ShowRequiredMark = true;
        lblCodeName.DisplayColon = true;
        lblSubject.ResourceString = "General.Subject";
        lblSubject.DisplayColon = true;
        lblText.ResourceString = "EmailTemplate_Edit.HTMLVersion";
        lblText.DisplayColon = true;
        lblTextPlain.ResourceString = "EmailTemplate_Edit.PlainTextVersion";
        lblTextPlain.DisplayColon = true;
        lblFrom.ResourceString = "General.FromEmail";
        lblFrom.DisplayColon = true;
        lblCc.ResourceString = "EmailTemplate_Edit.Cc";
        lblCc.DisplayColon = true;
        lblBcc.ResourceString = "EmailTemplate_Edit.Bcc";
        lblBcc.DisplayColon = true;
        AttachmentTitle.TitleText = ResHelper.GetString("general.attachments");

        // Register macro scripts
        this.RegisterMacroScript();

        // Save button
        string[,] actions = new string[1, 9];
        actions[0, 0] = HeaderActions.TYPE_SAVEBUTTON;
        actions[0, 1] = ResHelper.GetString("general.save");
        actions[0, 2] = null;
        actions[0, 3] = null;
        actions[0, 4] = null;
        actions[0, 5] = GetImageUrl("CMSModules/CMS_Content/EditMenu/save.png");
        actions[0, 6] = "lnksave_click";
        actions[0, 7] = null;
        actions[0, 8] = "true";

        this.CurrentMaster.HeaderActions.LinkCssClass = "ContentSaveLinkButton";
        this.CurrentMaster.HeaderActions.Actions = actions;
        this.CurrentMaster.HeaderActions.ActionPerformed += new CommandEventHandler(HeaderActions_ActionPerformed);

        string currentTemplate = ResHelper.GetString("EmailTemplate_Edit.CurrentTemplate");
        MacroResolver resolver = null;

        // Get 'templateid' from querystring
        templateId = QueryHelper.GetInteger("templateid", 0);
        if (templateId > 0)
        {
            // Get email template info of specified 'templateid'
            EmailTemplateInfo templateInfo = EmailTemplateProvider.GetEmailTemplate(templateId);
            // Get SiteID of the template
            siteId = templateInfo.TemplateSiteID;
            currentTemplate = templateInfo.TemplateDisplayName;

            // Setup macro resolvers
            resolver = MacroSelector.GetEmailTemplateResolver(templateInfo.TemplateName);
            macroSelectorElm.Resolver = resolver;
            macroSelectorPlain.Resolver = resolver;

            // Fill the form
            if (!RequestHelper.IsPostBack())
            {
                // Load data to the form
                LoadData(templateInfo);
                // Show message that the email template was created or updated successfully
                if (QueryHelper.GetBoolean("saved", false))
                {
                    lblInfo.Visible = true;
                    lblInfo.ResourceString = "General.ChangesSaved";
                }

                // Show attachment list
                AttachmentList.Visible = true;
                AttachmentTitle.Visible = true;
            }
        }
        else
        {
            // Get 'siteid' from querystring
            siteId = QueryHelper.GetInteger("siteid", 0);

            // Setup macro resolvers
            resolver = MacroSelector.GetEmailTemplateResolver(null);
            macroSelectorElm.Resolver = resolver;
            macroSelectorPlain.Resolver = resolver;
        }

        // Initialize page title control
        string[,] pageTitleTabs = new string[2, 3];
        pageTitleTabs[0, 0] = ResHelper.GetString("EmailTemplate_Edit.EmailTemplates");
        pageTitleTabs[0, 1] = "~/CMSSiteManager/Development/EmailTemplates/EmailTemplate_List.aspx?siteid=" + siteId.ToString();
        pageTitleTabs[0, 2] = String.Empty;
        pageTitleTabs[1, 0] = currentTemplate;
        pageTitleTabs[1, 1] = String.Empty;
        pageTitleTabs[1, 2] = String.Empty;

        // Init page title
        string imgUrl = null;
        string pageTitle = null;
        if (templateId > 0)
        {
            imgUrl = GetImageUrl("Objects/CMS_EmailTemplate/object.png");
            pageTitle = ResHelper.GetString("EmailTemplate_Edit.Title");
        }
        else
        {
            imgUrl = GetImageUrl("Objects/CMS_EmailTemplate/new.png");
            pageTitle = ResHelper.GetString("EmailTemplate_Edit.TitleNew");
        }

        this.CurrentMaster.Title.Breadcrumbs = pageTitleTabs;
        this.CurrentMaster.Title.TitleText = pageTitle;
        this.CurrentMaster.Title.TitleImage = imgUrl;
        this.CurrentMaster.Title.HelpTopicName = "newedit_e_mail_template";
        this.CurrentMaster.Title.HelpName = "helpTopic";
    }


    /// <summary>
    /// Registers scripts for proper "Insert macro" functionality.
    /// </summary>
    private void RegisterMacroScript()
    {
        StringBuilder builder = new StringBuilder();
        // Inserts the text to caret position (in text area for text or plain text)
        builder.AppendFormat("function InsertMacroPlain(text, obj) {{ InsertMacroGeneral(text, document.getElementById('{0}')); }}", txtPlainText.ClientID);
        builder.AppendLine();
        builder.AppendFormat("function InsertMacro(text, obj) {{ InsertMacroGeneral(text, document.getElementById('{0}')); }}", txtText.ClientID);

        ScriptHelper.RegisterClientScriptBlock(this.Page, typeof(string), "MacroInsert2", ScriptHelper.GetScript(builder.ToString()));
    }


    /// <summary>
    /// Load data of edited email template from DB into textboxes.
    /// </summary>
    /// <param name="templateInfo">EmailTemplateInfo object</param>
    protected void LoadData(EmailTemplateInfo templateInfo)
    {
        txtDisplayName.Text = templateInfo.TemplateDisplayName;
        txtCodeName.Text = templateInfo.TemplateName;
        txtSubject.Text = templateInfo.TemplateSubject;
        txtBcc.Text = templateInfo.TemplateBcc;
        txtCc.Text = templateInfo.TemplateCc;
        txtFrom.Text = templateInfo.TemplateFrom;
        txtText.Text = templateInfo.TemplateText;
        txtPlainText.Text = templateInfo.TemplatePlainText;

        // Init attachment storage
        AttachmentList.ObjectID = templateInfo.TemplateID;
        AttachmentList.ObjectType = EmailObjectType.EMAILTEMPLATE;
        AttachmentList.Category = MetaFileInfoProvider.OBJECT_CATEGORY_TEMPLATE;
        AttachmentList.AllowEdit = true;

        if (siteId != 0)
        {
            AttachmentList.SiteID = siteId;
        }
    }


    /// <summary>
    /// Save button action.
    /// </summary>
    protected void HeaderActions_ActionPerformed(object sender, CommandEventArgs e)
    {
        switch (e.CommandName.ToLower())
        {
            case "lnksave_click":
                txtDisplayName.Text = txtDisplayName.Text.Trim();
                txtCodeName.Text = txtCodeName.Text.Trim();
                txtSubject.Text = txtSubject.Text.Trim();

                // Find whether required fields are not empty
                string result = new Validator().NotEmpty(txtDisplayName.Text, ResHelper.GetString("EmailTemplate_Edit.FillDisplayNameField"))
                    .NotEmpty(txtCodeName.Text, ResHelper.GetString("EmailTemplate_Edit.FillCodeNameField"))
                    .IsCodeName(txtCodeName.Text, ResHelper.GetString("general.invalidcodename"))
                    .Result;

                // Check validity of entered e-mails
                if (!String.IsNullOrEmpty(this.txtFrom.Text) && !ValidationHelper.AreEmails(this.txtFrom.Text))
                {
                    result = ResHelper.GetString("EmailTemplate_Edit.InvalidFrom");
                }
                else if (!String.IsNullOrEmpty(this.txtBcc.Text) && !ValidationHelper.AreEmails(this.txtBcc.Text))
                {
                    result = ResHelper.GetString("EmailTemplate_Edit.InvalidBcc");
                }
                else if (!String.IsNullOrEmpty(this.txtCc.Text) && !ValidationHelper.AreEmails(this.txtCc.Text))
                {
                    result = ResHelper.GetString("EmailTemplate_Edit.InvalidCc");
                }

                if (String.IsNullOrEmpty(result))
                {
                    string siteName = null;
                    if (siteId != 0)
                    {
                        // Get site name for non-global templates
                        SiteInfo site = SiteInfoProvider.GetSiteInfo(siteId);
                        if (site != null)
                        {
                            siteName = site.SiteName;
                        }
                    }
                    // Try to get template by template name and site name
                    EmailTemplateInfo templateInfo = EmailTemplateProvider.GetEmailTemplate(txtCodeName.Text, siteName);

                    // Find if codename of the email template is unique for the site
                    if ((templateInfo == null) || (templateInfo.TemplateID == templateId) || ((templateInfo.TemplateSiteID == 0) && (siteId > 0)))
                    {
                        // Get object
                        if (templateInfo == null)
                        {
                            templateInfo = EmailTemplateProvider.GetEmailTemplate(templateId);
                            if (templateInfo == null)
                            {
                                templateInfo = new EmailTemplateInfo();
                            }
                        }

                        templateInfo.TemplateID = templateId;
                        templateInfo.TemplateDisplayName = txtDisplayName.Text;
                        templateInfo.TemplateName = txtCodeName.Text;
                        templateInfo.TemplateSubject = txtSubject.Text;
                        templateInfo.TemplateFrom = txtFrom.Text;
                        templateInfo.TemplateBcc = txtBcc.Text;
                        templateInfo.TemplateCc = txtCc.Text;
                        templateInfo.TemplateText = txtText.Text;
                        templateInfo.TemplatePlainText = txtPlainText.Text;
                        templateInfo.TemplateSiteID = siteId;

                        // Save EmailTemplateInfo object
                        EmailTemplateProvider.SetEmailTemplate(templateInfo);

                        UrlHelper.Redirect("EmailTemplate_Edit.aspx?templateid=" + templateInfo.TemplateID.ToString() + "&saved=1");
                    }
                    else
                    {
                        lblError.Visible = true;
                        lblError.ResourceString = "EmailTemplate_Edit.UniqueCodeName";
                    }
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = result;
                }
                break;
        }
    }

    #endregion
}
